<?php

namespace Crm\Plan\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Crm\Plan\Events\PlanAction as PlanActionEvent;
use Crm\Plan\Notifications\PlanAction as PlanActionNotification;
use Litepie\Actions\Concerns\AsAction;

class PlanAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the PlanActionEvent.
     *
     * @param   PlanActionEvent  $event
     * @return mixed
     */
    public function handle(PlanActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $plan.
     *
     * @param   PlanActionEvent  $event
     * @return void
     */
    public function create(PlanActionEvent $event)
    {
        $client = $event->plan->client;
        Notification::send($client, new PlanActionNotification($event));
    }

    /**
     * Handle the PlanActionEvent as a listener.
     *
     * @param   PlanActionEvent  $event
     * @return mixed
     */
    public function asListener(PlanActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   PlanActionEvent  $event
     * @return bool
     */
    private function isAllowed(PlanActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
