<?php

namespace Crm\Plan\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Crm\Plan\Events\PlanWorkflow as PlanWorkflowEvent;
use Crm\Plan\Notifications\PlanWorkflow as PlanWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class PlanWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(PlanWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $plan is submitted.
     *
     * @param PlanWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(PlanWorkflowEvent $event)
    {
        $client = $event->plan->client;
        Notification::send($client, new PlanWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $plan is published.
     *
     * @param PlanWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(PlanWorkflowEvent $event)
    {
        $client = $event->plan->client;
        Notification::send($client, new PlanWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $plan is approved.
     *
     * @param PlanWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(PlanWorkflowEvent $event)
    {
        $client = $event->plan->client;
        Notification::send($client, new PlanWorkflowNotification($event));
    }

    /**
     * Handles the $plan workflow event as a listener.
     *
     * @param PlanWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(PlanWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
