<?php

namespace Crm\Plan\Policies;

use Illuminate\Foundation\Auth\User as Authenticatable;
use Crm\Plan\Models\Plan;

class PlanPolicy
{


    /**
     * Determine if the given user can view the plan.
     *
     * @param Authenticatable $user
     * @param Plan $plan
     *
     * @return bool
     */
    public function view(Authenticatable $user, Plan $plan)
    {
        if ($authUser->canDo('plan.plan.view') && $authUser->isAdmin() || $user->isClient()) {
            return true;
        }

        return $plan->is_owner;
    }

    /**
     * Determine if the given user can create a plan.
     *
     * @param Authenticatable $user
     *
     * @return bool
     */
    public function create(Authenticatable $user)
    {
        return  $authUser->canDo('plan.plan.create');
    }

    /**
     * Determine if the given user can update the given plan.
     *
     * @param Authenticatable $user
     * @param Plan $plan
     *
     * @return bool
     */
    public function update(Authenticatable $user, Plan $plan)
    {
        if ($user->canDo('plan.plan.edit') && $user->isAdmin()) {
            return true;
        }

        return $plan->is_owner;
    }

    /**
     * Determine if the given user can delete the given plan.
     *
     * @param Authenticatable $user
     *
     * @return bool
     */
    public function destroy(Authenticatable $user, Plan $plan)
    {
        return $plan->is_owner;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
